from __future__ import absolute_import

from django.template import Library
from django.utils.safestring import mark_safe

from method_override import settings


__all__ = ['method_override']


register = Library()


@register.simple_tag
def method_override(method):
    '''Renders a hidden input with the method override value.'''
    input = settings.INPUT_TEMPLATE.format(
        name=settings.PARAM_KEY, value=method
    )
    return mark_safe(input)



from django.conf import settings


ALLOWED_HTTP_METHODS = getattr(
    settings,
    'METHOD_OVERRIDE_ALLOWED_HTTP_METHODS',
    ['GET', 'HEAD', 'PUT', 'POST', 'DELETE', 'OPTIONS', 'PATCH']
)

PARAM_KEY = getattr(
    settings,
    'METHOD_OVERRIDE_PARAM_KEY',
    '_method'
)

HTTP_HEADER = getattr(
    settings,
    'METHOD_OVERRIDE_HTTP_HEADER',
    'HTTP_X_HTTP_METHOD_OVERRIDE'
)

INPUT_TEMPLATE = getattr(
    settings,
    'METHOD_OVERRIDE_INPUT_TEMPLATE',
    '<input type="hidden" name="{name}" value="{value}">'
)



from method_override import settings


class MethodOverrideMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.method != 'POST':
            return self.get_response(request)
        method = self._get_method_override(request)
        if method in settings.ALLOWED_HTTP_METHODS:
            request.method = method
        return self.get_response(request)

    def _get_method_override(self, request):
        method = (
            request.POST.get(settings.PARAM_KEY) or
            request.META.get(settings.HTTP_HEADER)
        )
        return method and method.upper()



from os import path
from setuptools import find_packages, setup

from method_override import __version__

root = path.abspath(path.dirname(__file__))
with open(path.join(root, 'README.md'), encoding='utf-8') as file:
    long_description = file.read()

setup(
    name='django-method-override',
    version=__version__,
    description='Django Middleware for HTTP Method Override Form Params & Header',  # noqa
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='LocalMed',
    author_email='pete.browne@localmed.com',
    url='https://gitlab.com/localmed/django-method-override',
    license='MIT',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    install_requires=[
        'django>=2.0'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
    ],
)
